/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.basic.info.modular.usercollect.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unis.basic.info.modular.pricemonitor.entity.PriceMonitor;
import com.unis.basic.info.modular.pricemonitor.mapper.PriceMonitorMapper;
import com.unis.basic.info.modular.usercollect.entity.UserPrice;
import com.unis.basic.info.modular.usercollect.mapper.UserPriceMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.unis.common.enums.CommonSortOrderEnum;
import com.unis.common.exception.CommonException;
import com.unis.common.page.CommonPageRequest;
import com.unis.basic.info.modular.usercollect.entity.UserCollect;
import com.unis.basic.info.modular.usercollect.mapper.UserCollectMapper;
import com.unis.basic.info.modular.usercollect.param.UserCollectAddParam;
import com.unis.basic.info.modular.usercollect.param.UserCollectEditParam;
import com.unis.basic.info.modular.usercollect.param.UserCollectIdParam;
import com.unis.basic.info.modular.usercollect.param.UserCollectPageParam;
import com.unis.basic.info.modular.usercollect.service.UserCollectService;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 采集人员Service接口实现类
 *
 * @author weiyinbin
 * @date 2024/04/02 15:40
 **/
@Service
@Slf4j
public class UserCollectServiceImpl extends ServiceImpl<UserCollectMapper, UserCollect> implements UserCollectService {


    @Autowired
    private UserCollectMapper userCollectMapper;

    @Autowired
    private UserPriceMapper userPriceMapper;

    @Autowired
    private PriceMonitorMapper priceMonitorMapper;

    @Override
    public Page<UserCollect> page(UserCollectPageParam userCollectPageParam) {
        QueryWrapper<UserCollect> queryWrapper = new QueryWrapper<>();
        if (ObjectUtil.isAllNotEmpty(userCollectPageParam.getSortField(), userCollectPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(userCollectPageParam.getSortOrder());
            queryWrapper.orderBy(true, userCollectPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(userCollectPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(UserCollect::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    /**
     * @author: hujianchun
     * @Description: 新增采集人员信息
     * @param:
     * @return:
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(UserCollectAddParam userCollectAddParam) {

        //参数校验
        String phone = userCollectAddParam.getPhone();
        if (phone.length() != 11) {
            throw new CommonException("手机格式错误,请输入11位！");
        }

        UserCollect userCollect = new UserCollect();
        BeanUtils.copyProperties(userCollectAddParam, userCollect);
        userCollect.setCreateTime(new DateTime());
        userCollect.setUpdateTime(new DateTime());
        userCollect.setPassword("123456");
        userCollect.setAccount("123");
        int userCollectId = 0;
        try {
            userCollectId = userCollectMapper.insert(userCollect);
        } catch (Exception e) {
            log.error("新增监测采集人员信息失败,人员ID[{}],人员姓名[{}],异常信息内容[{}]", userCollect.getId(),
                    userCollect.getName(), e.getMessage());
            return;
        }

        //获取价格监测的id
        LambdaQueryWrapper<PriceMonitor> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PriceMonitor::getSysOrgId, userCollectAddParam.getSysOrgId());
        List<PriceMonitor> priceMonitors = priceMonitorMapper.selectList(wrapper);
        if (CollectionUtils.isEmpty(priceMonitors)) {
            log.error("获取价格监测信息异常,组织id[{}]", userCollectAddParam.getSysOrgId());
            throw new CommonException("获取价格监测信息异常,组织id[{}]", userCollectAddParam.getSysOrgId());
        }
        for (PriceMonitor priceMonitor : priceMonitors) {
            //插入中间表数据
            UserPrice userPrice = new UserPrice();
            userPrice.setUserCollectId(String.valueOf(userCollectId));
            userPrice.setPriceMonitorId(priceMonitor.getId());
            try {
                int insert2 = userPriceMapper.insert(userPrice);
                if (insert2 != 1) {
                    throw new CommonException("新增采集人员和价格监测关系失败");
                }
            } catch (Exception e) {
                log.error("新增采集人员和价格监测关系,数据库异常[{}]", e.getMessage());
                throw new CommonException("新增采集人员和价格监测关系失败");

            }
        }


    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(UserCollectEditParam userCollectEditParam) {
        UserCollect userCollect = this.queryEntity(userCollectEditParam.getId());
        BeanUtil.copyProperties(userCollectEditParam, userCollect);
        this.updateById(userCollect);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<UserCollectIdParam> userCollectIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(userCollectIdParamList, UserCollectIdParam::getId));
    }

    @Override
    public UserCollect detail(UserCollectIdParam userCollectIdParam) {
        return this.queryEntity(userCollectIdParam.getId());
    }

    @Override
    public UserCollect queryEntity(String id) {
        UserCollect userCollect = this.getById(id);
        if (ObjectUtil.isEmpty(userCollect)) {
            throw new CommonException("采集人员不存在，id值为：{}", id);
        }
        return userCollect;
    }

}
