/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.dev.modular.job.task;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.unis.basic.info.modular.grainPriceCollect.entity.GrainPriceCollect;
import com.unis.basic.info.modular.grainPriceCollect.mapper.GrainPriceCollectMapper;
import com.unis.basic.info.modular.grainPriceCollectCitySum.entity.GrainPriceCollectCitySum;
import com.unis.basic.info.modular.grainPriceCollectCitySum.mapper.GrainPriceCollectCitySumMapper;
import com.unis.basic.info.modular.pricemonitor.entity.PriceMonitor;
import com.unis.basic.info.modular.pricemonitor.mapper.PriceMonitorMapper;
import com.unis.basic.info.modular.remindcycle.entity.RemindCycle;
import com.unis.basic.info.modular.remindcycle.mapper.RemindCycleMapper;
import com.unis.basic.info.modular.reportpricemonitor.entity.ReportPriceMonitor;
import com.unis.basic.info.modular.reportpricemonitor.mapper.ReportPriceMonitorMapper;
import com.unis.basic.info.modular.unReporCity.entity.UnReporCity;
import com.unis.basic.info.modular.unReporCity.mapper.UnReporCityMapper;
import com.unis.common.exception.CommonException;
import com.unis.common.timer.CommonTimerTaskRunner;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 市州，站点漏报---定时任务
 *
 * @author xuyuxiang
 * @date 2022/8/5 15:52
 **/
@Slf4j
@Component
public class UnReportCityTaskRunner implements CommonTimerTaskRunner {

    @Resource
    private RemindCycleMapper remindCycleMapper;  //价格上报周期

    @Resource
    private GrainPriceCollectCitySumMapper grainPriceCollectCitySumMapper;  //市级粮食汇总

    @Resource
    private GrainPriceCollectMapper grainPriceCollectMapper; //站点汇总

    @Resource
    private ReportPriceMonitorMapper unreportPriceMonitorMapper; //站点漏报

    @Resource
    private UnReporCityMapper unReporCityMapper;  //市州漏报

    @Resource
    private PriceMonitorMapper priceMonitorMapper; //价格监测点


    //漏报：上报开始之前的信息和结束之后的信息，都属于漏报
    //错报：移动端采集人员进行登录采集的时候，若果上传超过两次就记为错报
    @Override
    public void action() {

        log.info("-----市州漏报的定时任务开始执行-----");


        //获取上报的周期信息 -周就一次上报
        List<RemindCycle> remindCycles = remindCycleMapper.selectList(
                new LambdaQueryWrapper<RemindCycle>()
                        .orderByDesc(RemindCycle::getCreateTime));
        if (CollectionUtils.isEmpty(remindCycles)) {
            throw new CommonException("暂无上报周期信息");
        }
        //获取到上报了的市州数据
        LambdaQueryWrapper<GrainPriceCollectCitySum> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(GrainPriceCollectCitySum::getStatus, 1);
        List<GrainPriceCollectCitySum> grainPriceCollectCitySums = grainPriceCollectCitySumMapper.selectList(wrapper);
        //获取上报了的站点数据
        LambdaQueryWrapper<GrainPriceCollect> priceCollectWrapper = new LambdaQueryWrapper<>();
        priceCollectWrapper.eq(GrainPriceCollect::getStatus, 0);
        List<GrainPriceCollect> grainPriceCollects = grainPriceCollectMapper.selectList(priceCollectWrapper);
        if (CollectionUtils.isEmpty(grainPriceCollects) && CollectionUtils.isEmpty(grainPriceCollectCitySums)) {
            log.info("没有需要检索的市州或站点漏报信息");
            return;
        }

        RemindCycle remindCycle = remindCycles.get(0);
        // 市级和站点都是同一个周期的
        //市州上报时间在周期开始之前或之后,都属于漏报
        if (CollectionUtil.isNotEmpty(grainPriceCollectCitySums)) {
            //过滤出有漏报市州的信息
            List<GrainPriceCollectCitySum> collect = grainPriceCollectCitySums.stream().filter(o -> {
                return o.getCreateTime().before(remindCycle.getStartTime())
                        || o.getCreateTime().after(remindCycle.getEndTime());
            }).collect(Collectors.toList());


            //创建map记录所在州市漏报集合,string为组织id
            HashMap<String, Integer> map = new HashMap<>();

            int unReporCityCount = 1;
            for (GrainPriceCollectCitySum grainPriceCollectCitySum : collect) {
                if (ObjectUtils.isEmpty(map.get(grainPriceCollectCitySum.getSysOrgId()))) {
                    map.put(grainPriceCollectCitySum.getSysOrgId(), unReporCityCount);
                    continue;
                }
                Integer count = map.get(grainPriceCollectCitySum.getSysOrgId());
                map.put(grainPriceCollectCitySum.getSysOrgId(), count + 1);
            }


            //插入对应表中
            Set<Map.Entry<String, Integer>> entrySet = map.entrySet();
            for (Map.Entry<String, Integer> entry : entrySet) {
                //记录到市州漏报表格中
                UnReporCity unReporCity = new UnReporCity();
                unReporCity.setCreateTime(new Date());
                unReporCity.setReportDate(new Date());
                unReporCity.setUpdateTime(new Date());
                unReporCity.setSysOrgId(entry.getKey());
                unReporCity.setUnSelfPriceMonitorCount(0);
                unReporCity.setUnReportCount(entry.getValue());
                List<UnReporCity> unReporCities = unReporCityMapper.selectList(Wrappers.lambdaQuery(UnReporCity.class).eq(UnReporCity::getSysOrgId, entry.getKey()));
                if (!CollectionUtils.isEmpty(unReporCities)) {
                    UnReporCity reporCity = unReporCities.get(0);
                    try {
                        unReporCityMapper.update(null, new LambdaUpdateWrapper<UnReporCity>()
                                .eq(UnReporCity::getId, reporCity.getId())
                                .set(UnReporCity::getUnReportCount, reporCity.getUnReportCount() + 1));
                    } catch (Exception e) {
                        log.error("更新新增站点漏报记录失败,堆栈信息[{}]", e);
                        throw new CommonException("更新市州漏报新增数据失败");
                    }
                } else {
                    try {
                        unReporCityMapper.insert(unReporCity);
                    } catch (Exception e) {
                        log.error("新增站点漏报记录失败,堆栈信息[{}]", e);
                        throw new CommonException("市州漏报新增数据失败");
                    }
                }
            }
        }
        if (CollectionUtil.isNotEmpty(grainPriceCollects)) {

            List<GrainPriceCollect> collect = grainPriceCollects.stream().filter(o -> {
                        return o.getUserCollectDate().before(remindCycle.getStartTime()) ||
                                o.getUserCollectDate().after(remindCycle.getEndTime());
                    }
            ).collect(Collectors.toList());

            HashMap<String, Integer> map = new HashMap<>();
            int unReporCityCount = 1;
            for (GrainPriceCollect grainPriceCollect : collect) {
                if (ObjectUtils.isEmpty(map.get(grainPriceCollect.getPriceMonitorId()))) {
                    map.put(grainPriceCollect.getPriceMonitorId(), unReporCityCount);
                    continue;
                }
                Integer count = map.get(grainPriceCollect.getPriceMonitorId());
                map.put(grainPriceCollect.getPriceMonitorId(), count + 1);

            }

            HashMap<Integer, Integer> unSelfPriceMonitorCountMap = new HashMap<>();
            int grainPriceCollectCount=1;
            for (GrainPriceCollect grainPriceCollect : collect) {
                if (ObjectUtils.isEmpty(map.get(grainPriceCollect.getIsSelfPriceMonitor()))) {
                    unSelfPriceMonitorCountMap.put(grainPriceCollect.getIsSelfPriceMonitor(), grainPriceCollectCount);
                    continue;
                }
                Integer count = map.get(grainPriceCollect.getIsSelfPriceMonitor());
                unSelfPriceMonitorCountMap.put(grainPriceCollect.getIsSelfPriceMonitor(), count + 1);

            }

            Set<Map.Entry<String, Integer>> entries = map.entrySet();
            for (Map.Entry<String, Integer> entry : entries) {
                //获取站点id
                String priceMonitorId = entry.getKey();
                List<PriceMonitor> priceMonitors = priceMonitorMapper.selectList(new LambdaQueryWrapper<PriceMonitor>()
                        .eq(PriceMonitor::getId, priceMonitorId));
                if (CollectionUtil.isEmpty(priceMonitors)) {
                    log.error("未找到对应的站点信息，站点id[{}]", priceMonitorId);
                    continue;
                }

                PriceMonitor priceMonitor = priceMonitors.get(0);
                //获取对应州市id
                String sysOrgId = priceMonitor.getSysOrgId();
                ReportPriceMonitor reportPriceMonitor = new ReportPriceMonitor();
                reportPriceMonitor.setCreateTime(new Date());
                reportPriceMonitor.setUpdateTime(new Date());
                reportPriceMonitor.setUnReportCityId(sysOrgId);
                reportPriceMonitor.setPriceMonitorId(entry.getKey());
                reportPriceMonitor.setUnReportCount(entry.getValue());
                reportPriceMonitor.setReportDate(new Date());
                //移动端的数据
                reportPriceMonitor.setUnSelfPriceMonitorCount(grainPriceCollectCount);

                try {
                    unreportPriceMonitorMapper.insert(reportPriceMonitor);
                } catch (Exception e) {
                    log.error("新增站点漏报记录失败,堆栈信息[{}]", e);
                    throw new CommonException("站点漏报新增数据失败");
                }
            }
        }
    }


}



